'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var min = require('libphonenumber-js/min');
var React = _interopDefault(require('react'));

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
}

/**
 * Headless phone number input formatter.
 */
function PhoneFormatter(_a) {
    var _b;
    var { onChange } = _a, props = __rest(_a, ["onChange"]);
    const [inputValue, setInputValue] = React.useState("");
    const [formatter] = React.useState(() => new min.AsYouType(props.defaultCountry));
    const [impossible, setImpossible] = React.useState(null);
    function setValue(newValue) {
        var _a;
        if (inputValue == newValue)
            return;
        if (
        // check if this is just appending a value
        newValue.length > inputValue.length &&
            newValue.slice(0, inputValue.length) == inputValue) {
            setInputValue(formatter.input(newValue.slice(inputValue.length)));
        }
        else {
            formatter.reset();
            formatter.input(newValue);
            setInputValue(newValue);
        }
        onChange((_a = formatter.getNumber()) === null || _a === void 0 ? void 0 : _a.number);
    }
    React.useLayoutEffect(() => {
        var _a, _b, _c;
        const number = (_a = formatter.getNumber()) === null || _a === void 0 ? void 0 : _a.number;
        if (number) {
            if (number != props.value) {
                // override the phone number if the field has a number and it doesn't match the prop value.
                formatter.reset();
                setInputValue(props.value ? formatter.input(props.value) : "");
            }
            else {
                // no change, input value matches field value
                return;
            }
        }
        else if (props.value) {
            // no valid number in the field, but prop value is set
            // treat as input
            formatter.reset();
            setInputValue(formatter.input(props.value));
        }
        else {
            // No valid number in the field and no prop value - no change.
            return;
        }
        setImpossible(((_b = formatter.getNumber()) === null || _b === void 0 ? void 0 : _b.isPossible()) === false);
        onChange((_c = formatter.getNumber()) === null || _c === void 0 ? void 0 : _c.number);
    }, [props.value, formatter, inputValue, onChange]);
    const country = ((_b = formatter.getNumber()) === null || _b === void 0 ? void 0 : _b.country) || props.defaultCountry;
    return (React.createElement(React.Fragment, null, props.children({
        country: country,
        impossible: impossible,
        inputValue,
        onChange(e) {
            var _a;
            setValue(e.target.value);
            if (impossible && ((_a = formatter.getNumber()) === null || _a === void 0 ? void 0 : _a.isPossible()))
                setImpossible(false);
        },
        onBlur() {
            const number = formatter.getNumber();
            if (number) {
                const possible = number.isPossible();
                setImpossible(!possible);
                if (possible) {
                    const e164 = number.number;
                    formatter.reset();
                    setInputValue(formatter.input(e164));
                    onChange(e164);
                }
                else {
                    setInputValue(formatter.input(""));
                }
            }
        },
    })));
}

module.exports = PhoneFormatter;
